<?php
/**
 * This file is part of phpDocumentor.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @copyright 2010-2016 Mike van Riel<mike@phpdoc.org>
 * @license   http://www.opensource.org/licenses/mit-license.php MIT
 * @link      http://phpdoc.org
 */

namespace phpDocumentor\DomainModel\Renderer;

use phpDocumentor\DomainModel\Path;

/**
 * A single piece of output generated by the Renderer such as a HTML page or PDF.
 */
final class Artefact
{
    /** @var Path Where to store the Artefact relative to the destination specified to the renderer */
    private $location;

    /** @var string the contents of this artefact */
    private $content;

    /**
     * @param Path $location
     * @param string $content
     */
    public function __construct(Path $location, $content)
    {
        $this->location = $location;
        $this->content = $content;
    }

    /**
     * Returns the location where this artefact is supposed to be stored relative.
     *
     * @return Path
     */
    public function location()
    {
        return $this->location;
    }

    /**
     * Returns the artefact's contents.
     *
     * @return string
     */
    public function content()
    {
        return $this->content;
    }
}
